{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Core.Classes.Callback2
       (
         Callback2,
         Callback2Class(..),
         callback2,
         callback2_create,
         callback2_inputShape,
         callback2_inputSparsity,
         callback2_nIn,
         callback2_nOut,
         callback2_name,
         callback2_operator_call,
         callback2_options,
         callback2_outputShape,
         callback2_outputSparsity,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import qualified Data.Map as M
import Foreign.C.Types
import Foreign.Marshal ( new, free )
import Foreign.Storable ( peek )
import Foreign.Ptr ( Ptr, nullPtr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Internal.FormatException ( formatException )
import Casadi.Internal.MarshalTypes ( StdVec, StdString, StdMap, StdPair ) -- StdPair StdOstream'
import Casadi.Internal.Marshal ( Marshal(..), withMarshal )
import Casadi.Internal.WrapReturn ( WrapReturn(..) )
import Casadi.Core.Data
import Casadi.Core.Enums
-- direct wrapper
foreign import ccall unsafe "casadi__Callback2__CONSTRUCTOR" c_casadi__Callback2__CONSTRUCTOR
  :: Ptr (Ptr StdString) -> IO (Ptr Callback2')
casadi__Callback2__CONSTRUCTOR
  :: IO Callback2
casadi__Callback2__CONSTRUCTOR  =

  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__Callback2__CONSTRUCTOR errStrPtrP 
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
callback2 :: IO Callback2
callback2 = casadi__Callback2__CONSTRUCTOR


-- direct wrapper
foreign import ccall unsafe "casadi__Callback2__create" c_casadi__Callback2__create
  :: Ptr (Ptr StdString) -> Ptr Callback2' -> IO (Ptr Function')
casadi__Callback2__create
  :: Callback2 -> IO Function
casadi__Callback2__create x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__Callback2__create errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
callback2_create :: Callback2Class a => a -> IO Function
callback2_create x = casadi__Callback2__create (castCallback2 x)


-- direct wrapper
foreign import ccall unsafe "casadi__Callback2__inputShape" c_casadi__Callback2__inputShape
  :: Ptr (Ptr StdString) -> Ptr Callback2' -> CInt -> IO (Ptr (StdPair CInt CInt))
casadi__Callback2__inputShape
  :: Callback2 -> Int -> IO (Int, Int)
casadi__Callback2__inputShape x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__Callback2__inputShape errStrPtrP x0' x1'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
callback2_inputShape :: Callback2Class a => a -> Int -> IO (Int, Int)
callback2_inputShape x = casadi__Callback2__inputShape (castCallback2 x)


-- direct wrapper
foreign import ccall unsafe "casadi__Callback2__inputSparsity" c_casadi__Callback2__inputSparsity
  :: Ptr (Ptr StdString) -> Ptr Callback2' -> CInt -> IO (Ptr Sparsity')
casadi__Callback2__inputSparsity
  :: Callback2 -> Int -> IO Sparsity
casadi__Callback2__inputSparsity x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__Callback2__inputSparsity errStrPtrP x0' x1'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
callback2_inputSparsity :: Callback2Class a => a -> Int -> IO Sparsity
callback2_inputSparsity x = casadi__Callback2__inputSparsity (castCallback2 x)


-- direct wrapper
foreign import ccall unsafe "casadi__Callback2__nIn" c_casadi__Callback2__nIn
  :: Ptr (Ptr StdString) -> Ptr Callback2' -> IO CInt
casadi__Callback2__nIn
  :: Callback2 -> IO Int
casadi__Callback2__nIn x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__Callback2__nIn errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
callback2_nIn :: Callback2Class a => a -> IO Int
callback2_nIn x = casadi__Callback2__nIn (castCallback2 x)


-- direct wrapper
foreign import ccall unsafe "casadi__Callback2__nOut" c_casadi__Callback2__nOut
  :: Ptr (Ptr StdString) -> Ptr Callback2' -> IO CInt
casadi__Callback2__nOut
  :: Callback2 -> IO Int
casadi__Callback2__nOut x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__Callback2__nOut errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
callback2_nOut :: Callback2Class a => a -> IO Int
callback2_nOut x = casadi__Callback2__nOut (castCallback2 x)


-- direct wrapper
foreign import ccall unsafe "casadi__Callback2__name" c_casadi__Callback2__name
  :: Ptr (Ptr StdString) -> Ptr Callback2' -> IO (Ptr StdString)
casadi__Callback2__name
  :: Callback2 -> IO String
casadi__Callback2__name x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__Callback2__name errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
callback2_name :: Callback2Class a => a -> IO String
callback2_name x = casadi__Callback2__name (castCallback2 x)


-- direct wrapper
foreign import ccall unsafe "casadi__Callback2__operator_call" c_casadi__Callback2__operator_call
  :: Ptr (Ptr StdString) -> Ptr Callback2' -> Ptr (StdVec (Ptr DMatrix')) -> IO (Ptr (StdVec (Ptr DMatrix')))
casadi__Callback2__operator_call
  :: Callback2 -> Vector DMatrix -> IO (Vector DMatrix)
casadi__Callback2__operator_call x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__Callback2__operator_call errStrPtrP x0' x1'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
callback2_operator_call :: Callback2Class a => a -> Vector DMatrix -> IO (Vector DMatrix)
callback2_operator_call x = casadi__Callback2__operator_call (castCallback2 x)


-- direct wrapper
foreign import ccall unsafe "casadi__Callback2__options" c_casadi__Callback2__options
  :: Ptr (Ptr StdString) -> Ptr Callback2' -> IO (Ptr (StdMap StdString (Ptr GenericType')))
casadi__Callback2__options
  :: Callback2 -> IO (M.Map String GenericType)
casadi__Callback2__options x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__Callback2__options errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
callback2_options :: Callback2Class a => a -> IO (M.Map String GenericType)
callback2_options x = casadi__Callback2__options (castCallback2 x)


-- direct wrapper
foreign import ccall unsafe "casadi__Callback2__outputShape" c_casadi__Callback2__outputShape
  :: Ptr (Ptr StdString) -> Ptr Callback2' -> CInt -> IO (Ptr (StdPair CInt CInt))
casadi__Callback2__outputShape
  :: Callback2 -> Int -> IO (Int, Int)
casadi__Callback2__outputShape x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__Callback2__outputShape errStrPtrP x0' x1'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
callback2_outputShape :: Callback2Class a => a -> Int -> IO (Int, Int)
callback2_outputShape x = casadi__Callback2__outputShape (castCallback2 x)


-- direct wrapper
foreign import ccall unsafe "casadi__Callback2__outputSparsity" c_casadi__Callback2__outputSparsity
  :: Ptr (Ptr StdString) -> Ptr Callback2' -> CInt -> IO (Ptr Sparsity')
casadi__Callback2__outputSparsity
  :: Callback2 -> Int -> IO Sparsity
casadi__Callback2__outputSparsity x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__Callback2__outputSparsity errStrPtrP x0' x1'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
callback2_outputSparsity :: Callback2Class a => a -> Int -> IO Sparsity
callback2_outputSparsity x = casadi__Callback2__outputSparsity (castCallback2 x)

