{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Core.Classes.DleSolver
       (
         DleSolver,
         DleSolverClass(..),
         dleSolver__0,
         dleSolver__1,
         dleSolver__2,
         dleSolver_clone,
         dleSolver_doc,
         dleSolver_getSparsity,
         dleSolver_hasPlugin,
         dleSolver_loadPlugin,
         dleSolver_printStats,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import qualified Data.Map as M
import Foreign.C.Types
import Foreign.Marshal ( new, free )
import Foreign.Storable ( peek )
import Foreign.Ptr ( Ptr, nullPtr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Internal.FormatException ( formatException )
import Casadi.Internal.MarshalTypes ( StdVec, StdString, StdMap, StdPair ) -- StdPair StdOstream'
import Casadi.Internal.Marshal ( Marshal(..), withMarshal )
import Casadi.Internal.WrapReturn ( WrapReturn(..) )
import Casadi.Core.Data
import Casadi.Core.Enums
-- direct wrapper
foreign import ccall unsafe "casadi__DleSolver__CONSTRUCTOR__0" c_casadi__DleSolver__CONSTRUCTOR__0
  :: Ptr (Ptr StdString) -> Ptr StdString -> Ptr StdString -> Ptr (StdMap StdString (Ptr Sparsity')) -> IO (Ptr DleSolver')
casadi__DleSolver__CONSTRUCTOR__0
  :: String -> String -> M.Map String Sparsity -> IO DleSolver
casadi__DleSolver__CONSTRUCTOR__0 x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__DleSolver__CONSTRUCTOR__0 errStrPtrP x0' x1' x2'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
dleSolver__0 :: String -> String -> M.Map String Sparsity -> IO DleSolver
dleSolver__0 = casadi__DleSolver__CONSTRUCTOR__0


-- direct wrapper
foreign import ccall unsafe "casadi__DleSolver__CONSTRUCTOR__1" c_casadi__DleSolver__CONSTRUCTOR__1
  :: Ptr (Ptr StdString) -> Ptr StdString -> Ptr StdString -> Ptr (StdMap StdString (Ptr Sparsity')) -> Ptr (StdMap StdString (Ptr GenericType')) -> IO (Ptr DleSolver')
casadi__DleSolver__CONSTRUCTOR__1
  :: String -> String -> M.Map String Sparsity -> M.Map String GenericType -> IO DleSolver
casadi__DleSolver__CONSTRUCTOR__1 x0 x1 x2 x3 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  withMarshal x3 $ \x3' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__DleSolver__CONSTRUCTOR__1 errStrPtrP x0' x1' x2' x3'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
dleSolver__1 :: String -> String -> M.Map String Sparsity -> M.Map String GenericType -> IO DleSolver
dleSolver__1 = casadi__DleSolver__CONSTRUCTOR__1


-- direct wrapper
foreign import ccall unsafe "casadi__DleSolver__CONSTRUCTOR__2" c_casadi__DleSolver__CONSTRUCTOR__2
  :: Ptr (Ptr StdString) -> IO (Ptr DleSolver')
casadi__DleSolver__CONSTRUCTOR__2
  :: IO DleSolver
casadi__DleSolver__CONSTRUCTOR__2  =

  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__DleSolver__CONSTRUCTOR__2 errStrPtrP 
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
dleSolver__2 :: IO DleSolver
dleSolver__2 = casadi__DleSolver__CONSTRUCTOR__2


-- direct wrapper
foreign import ccall unsafe "casadi__DleSolver__clone" c_casadi__DleSolver__clone
  :: Ptr (Ptr StdString) -> Ptr DleSolver' -> IO (Ptr DleSolver')
casadi__DleSolver__clone
  :: DleSolver -> IO DleSolver
casadi__DleSolver__clone x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__DleSolver__clone errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
dleSolver_clone :: DleSolverClass a => a -> IO DleSolver
dleSolver_clone x = casadi__DleSolver__clone (castDleSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__DleSolver__doc" c_casadi__DleSolver__doc
  :: Ptr (Ptr StdString) -> Ptr StdString -> IO (Ptr StdString)
casadi__DleSolver__doc
  :: String -> IO String
casadi__DleSolver__doc x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__DleSolver__doc errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
dleSolver_doc :: String -> IO String
dleSolver_doc = casadi__DleSolver__doc


-- direct wrapper
foreign import ccall unsafe "casadi__DleSolver__getSparsity" c_casadi__DleSolver__getSparsity
  :: Ptr (Ptr StdString) -> Ptr (StdMap StdString (Ptr Sparsity')) -> IO (Ptr Sparsity')
casadi__DleSolver__getSparsity
  :: M.Map String Sparsity -> IO Sparsity
casadi__DleSolver__getSparsity x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__DleSolver__getSparsity errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
dleSolver_getSparsity :: M.Map String Sparsity -> IO Sparsity
dleSolver_getSparsity = casadi__DleSolver__getSparsity


-- direct wrapper
foreign import ccall unsafe "casadi__DleSolver__hasPlugin" c_casadi__DleSolver__hasPlugin
  :: Ptr (Ptr StdString) -> Ptr StdString -> IO CInt
casadi__DleSolver__hasPlugin
  :: String -> IO Bool
casadi__DleSolver__hasPlugin x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__DleSolver__hasPlugin errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
dleSolver_hasPlugin :: String -> IO Bool
dleSolver_hasPlugin = casadi__DleSolver__hasPlugin


-- direct wrapper
foreign import ccall unsafe "casadi__DleSolver__loadPlugin" c_casadi__DleSolver__loadPlugin
  :: Ptr (Ptr StdString) -> Ptr StdString -> IO ()
casadi__DleSolver__loadPlugin
  :: String -> IO ()
casadi__DleSolver__loadPlugin x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__DleSolver__loadPlugin errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
dleSolver_loadPlugin :: String -> IO ()
dleSolver_loadPlugin = casadi__DleSolver__loadPlugin


-- direct wrapper
foreign import ccall unsafe "casadi__DleSolver__printStats" c_casadi__DleSolver__printStats
  :: Ptr (Ptr StdString) -> Ptr DleSolver' -> IO ()
casadi__DleSolver__printStats
  :: DleSolver -> IO ()
casadi__DleSolver__printStats x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__DleSolver__printStats errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
dleSolver_printStats :: DleSolverClass a => a -> IO ()
dleSolver_printStats x = casadi__DleSolver__printStats (castDleSolver x)

